/*
 * [The "BSD licence"]
 * Copyright (c) 2012 Dandelion
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Dandelion nor the names of its contributors 
 * may be used to endorse or promote products derived from this software 
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.github.dandelion.datatables.core.extension;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.github.dandelion.datatables.core.asset.JsResource;
import com.github.dandelion.datatables.core.asset.WebResources;
import com.github.dandelion.datatables.core.configuration.Configuration;
import com.github.dandelion.datatables.core.exception.BadConfigurationException;
import com.github.dandelion.datatables.core.exception.ExtensionLoadingException;
import com.github.dandelion.datatables.core.html.HtmlTable;
import com.github.dandelion.datatables.core.util.ClassUtils;
import com.github.dandelion.datatables.core.util.StringUtils;

/**
 * Extension manager.
 *
 * @author Thibault Duchateau
 */
public class ExtensionManager {

	// Logger
	private Logger logger = LoggerFactory.getLogger(this.getClass());
		
	private HtmlTable table;
	
	public ExtensionManager(HtmlTable table){
		this.table = table;
	}
	
	public void loadExtensions(JsResource mainJsFile, Map<String, Object> mainConf, WebResources webResources) throws ExtensionLoadingException {
		
		registerCustomExtensions(table);
		
		ExtensionLoader extensionLoader = new ExtensionLoader(table, mainJsFile, mainConf, webResources);
		extensionLoader.load(table.getTableConfiguration().getInternalExtensions());
		if(table.getTableConfiguration().getCssTheme() != null){
			extensionLoader.load(new HashSet<Extension>(Arrays.asList(table.getTableConfiguration().getCssTheme())));			
		}
	}
	
	/**
	 * Add custom extensions (for now features and plugins) to the current table
	 * if they're activated.
	 * 
	 * @param table
	 *            the HtmlTable to update with custom extensions.
	 * @throws BadConfigurationException
	 */
	public void registerCustomExtensions(HtmlTable table) throws ExtensionLoadingException {

		if (StringUtils.isNotBlank(table.getTableConfiguration().getMainExtensionPackage())) {

			logger.debug("Scanning custom extensions...");

			// Scanning custom extension based on the base.package property
			List<Extension> customExtensions = scanCustomExtensions(table.getTableConfiguration().getMainExtensionPackage());

			// Load custom extension if enabled
			if (customExtensions != null && !customExtensions.isEmpty() && table.getTableConfiguration().getMainExtensionNames() != null) {
				for (String extensionToRegister : table.getTableConfiguration().getMainExtensionNames()) {
					for (Extension customExtension : customExtensions) {
						if (extensionToRegister.equals(customExtension.getName().toLowerCase())) {
							table.getTableConfiguration().registerExtension(customExtension);
							logger.debug("Custom extension {} registered", customExtension.getName());
							continue;
						}
					}
				}
			} else {
				logger.warn("A base backage to scan has been detected but no custom extension has been found");
			}
		}
		else{
			logger.debug("The {} property is blank. Unable to scan any class.", Configuration.MAIN_EXTENSION_PACKAGE.getName());
		}
	}
	
	/**
	 * 
	 * @param packageName
	 * @return
	 * @throws ExtensionLoadingException
	 */
	public List<Extension> scanCustomExtensions(String packageName) throws ExtensionLoadingException {

		// Init return value
		List<Extension> retval = new ArrayList<Extension>();

		List<Class<?>> customExtensionClassList = null;
		
		try {
			customExtensionClassList = ClassUtils.getSubClassesInPackage(packageName, AbstractExtension.class);
		} catch (ClassNotFoundException e) {
			throw new ExtensionLoadingException("Unable to load custom extensions", e);
		} catch (IOException e) {
			throw new ExtensionLoadingException("Unable to access the package '" + packageName + "'", e);
		}
		
		// Instanciate all found classes
		for (Class<?> clazz : customExtensionClassList) {

			try {
				retval.add((AbstractExtension) ClassUtils.getClass(clazz.getName()).newInstance());
			} catch (InstantiationException e) {
				throw new ExtensionLoadingException("Unable to instanciate the class " + clazz.getName(), e);
			} catch (IllegalAccessException e) {
				throw new ExtensionLoadingException("Unable to access the class " + clazz.getName(), e);
			} catch (ClassNotFoundException e) {
				throw new ExtensionLoadingException("Unable to load the class " + clazz.getName(), e);
			}
		}

		return retval;
	}
}